<?php

/**
 * @file
 * Contains \Drupal\term_condition\Plugin\Condition\Term.
 */

namespace Drupal\term_condition\Plugin\Condition;

use Drupal\Core\Condition\ConditionPluginBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\Context\ContextDefinition;

/**
* Provides a 'Term' condition to enable a condition based in module selected status.
*
* @Condition(
*   id = "term",
*   label = @Translation("Term"),
*   context = {
*     "node" = @ContextDefinition("entity:node", required = FALSE , label = @Translation("node"))
*   }
* )
*
*/
class Term extends ConditionPluginBase {
  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition)
  {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition
    );
  }

  /**
   * Creates a new ExampleCondition instance.
   *
   * @param array $configuration
   *   The plugin configuration, i.e. an array with configuration values keyed
   *   by configuration option name. The special key 'context' may be used to
   *   initialize the defined contexts by setting it to an array of context
   *   values keyed by context names.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $terms = $this->configuration['tid'];
    $default_terms = [];
    if(!empty($terms)) {

      if (is_array($terms)) {
        foreach ($terms as $key => $term) {
          $term = array_pop($term);
          $default_terms[] = \Drupal\taxonomy\Entity\Term::load($term);
        }
      } else {
        $default_terms = \Drupal\taxonomy\Entity\Term::load($terms);
      }
    }

    $form['tid'] = array(
      '#type' => 'entity_autocomplete',
      '#title' => $this->t('Select taxonomy term(s)'),
      '#default_value' => $default_terms,
      '#target_type' => 'taxonomy_term',
      '#tags' => TRUE,
    );

    return parent::buildConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->configuration['tid'] = $form_state->getValue('tid');
    parent::submitConfigurationForm($form, $form_state);
  }

  /**
   * Evaluates the condition and returns TRUE or FALSE accordingly.
   *
   * @return bool
   *   TRUE if the condition has been met, FALSE otherwise.
   */
  public function evaluate() {
    if (empty($this->configuration['tid']) && !$this->isNegated()) {
      return TRUE;
    }

    // If configuration['tid'] is an array, there is multiple terms set.
    if(is_array($this->configuration['tid'])) {
      $tids = $this->configuration['tid'];
      unset($this->configuration['tid']);
      foreach ($tids as $tid) {
        $this->configuration['tid'][] = array_pop($tid);
      }
    }

    $node = $this->getContextValue('node');

    // Not in a node context.
    if (!$node) {
      return FALSE;
    }

    foreach ($node->referencedEntities() as $referenced_entity) {
      // If configuration['tid'] is an array with multiple terms, check all
      // tids in the array against the term.
      if(is_array($this->configuration['tid'])) {
        if ($referenced_entity->getEntityTypeId() == 'taxonomy_term'
          && in_array($referenced_entity->id(), $this->configuration['tid'])) {
          return TRUE;
        }
      }
      else {
        if ($referenced_entity->getEntityTypeId() == 'taxonomy_term'
          && $referenced_entity->id() == $this->configuration['tid']) {
          return TRUE;
        }
      }
    }

    return FALSE;
  }

  /**
   * Provides a human readable summary of the condition's configuration.
   */
  public function summary()
  {
    $tid = $this->configuration['tid'];

    if (!empty($this->configuration['negate'])) {
      return $this->t('The node is not associated with taxonomy term @tid.', array('@tid' => $tid));
    }
    else {
      return $this->t('The node is associated with taxonomy term @tid.', array('@tid' => $tid));
    }
 }

}
